include <imhdr.h>

# The following diagrams show the CCD geometry for the different readout modes.
#
# Single readout:
#				G
#		<-------------------------------->
#			    A		    F	 
#		<----------------------><-------->
#		 B 	     	     C   D      E
#		<->                 <--><>      <>
#	^   ^	+-----------------------+--------+  ^	^
#	|   |	|   		     	| 	 |  | e	|
#	| c |	|   		     	|--------|  v  	|
#	|   v	|  +----------------+   ||      ||      |
#	|    	|  |		    |	||  o	||    	|
#	|    	|  |		    |	||  v	||    	|
#	|    	|  |		    |	||  e	||    	|
#     a	|    	|  |		    |	||  r	||      | f
#	|    	|  |		    |	||  s	||    	|
#	|    	|  |		    |	||  c	||    	|
#	|    	|  |		    |	||  a	||    	|
#	|    	|  +----------------+   ||  n   ||    	|
#	|   ^	|   		     	|--------|   	|
#	| b |   |                       |        |  ^	|
#	|   |	|   		     	| 	 |  | d	|
#	v   V	*-----------------------+--------+  v	v
#
# Quad readout (single frame):
#				G'
#		<--------------------------------->
#		     A/2    F/2  F/2      A/2
#		<---------><---><---><----------->
#		 B 	   D   2*E   D         C
#		<->        <>  < >  <> 	     <-->
#	^   ^	*----------+----.----+------------* ^	^
#	|   |	|   	   |    .    |            | | e	|
#	| c |	|   	   |----.----|            | v	|
#	|   v	|  +-------+|  |.|  |+--------+   |   	|
#  a/2	|    	|  |	   ||o |.|o ||        |   |  	| f/2
#	|    	|  |   3   ||v |.|v ||   4    |   |   	|
#	|    	|  |	   ||  |.|  ||        |   |  	|
#	|    	|  |	   ||3 |.|4 ||        |   |  	|
#       V    	...................................   	v
#	^    	|  |	   ||o |.|o ||        |   |  	^
#	|    	|  |   1   ||v |.|v ||   2    |   |  	|
#	|    	|  |	   ||  |.|  ||        |   |   	|
#   a/2	|    	|  +-------+|1 |.|2 |+--------+   |  	| f/2
#	|   ^	|          ||  |.|  ||            |  	|
#	| b |	|   	   |----.----|            | ^	|
#	|   |	|   	   |    .    |            | | d	|
#	v   v	*----------+----.----+------------* v	V
#
#
# Quad readout (four frames):
#
#		       G"			G"
#		<-------------->        <---------------->
#		     A/2     F/2         F/2      A/2
#		<---------><---->       <----><---------->
#		 B 	   D   E        E   D          C
#		<->        <>  <>	<>  <>	      <-->
#	^   ^	*----------+----+	+----+------------* ^	^
#	|   |	|   	   |    |	|    |            | | e	|
#	| c |	|   	   |----|	|----|            | v	|
#   a/2	|   v	|  +-------+|  ||	||  |+--------+   | ^ 	|
#	|    	|  |	   ||o ||	||o ||        |   | |	| f/2
#	|    	|  |   3   ||v ||	||v ||   4    |   | |	|
#	|    	|  |	   ||  ||	||  ||        |   | |	|
#	V    	|  |	   ||3 ||	||4 ||        |   | |	|
#		+---------------+       +-----------------+ v	v
#
#
#            	+---------------+	+-----------------+ ^	^
#	^    	|  |	   ||o ||	||o ||        |   | |	|
#	|    	|  |   1   ||v ||	||v ||   2    |   | |	|
#	|    	|  |	   ||  ||	||  ||        |   | |	|
#	|    	|  +-------+|1 ||	||2 |+--------+   | |	| f/2
#   a/2	|   ^	|          ||  ||	||  ||            | v	|
#	| b |	|   	   |----|	|----|            | ^	|
#	|   |	|   	   |    |	|    |            | | d	|
#	v   v	*----------+----+	+----+------------* v	v
#
# Where
#		A  = xdata			a = ydata
#		B  = txskip1			b = tyskip1
#		C  = txskip2			c = tyskip2
#		D  = bxskip1			d = byskip1
#		E  = bxskip2			e = byskip2
#		F  = xover			f = yover = a
#		G  =  A + F
#		G' =  A + F
#		G" = (A + F) / 2

procedure quadinit (im)

pointer	im			#I Pointer to input image

int	quadrant		#I Quadrant number
pointer	quad			#I Pointer to quadrant image

int	nx, ny, xdata, ydata, xover, yover
char	datasec[SZ_LINE]
int	dx1, dx2, dxs, dy1, dy2, dys
int	ddx1, ddx2, ddy1, ddy2
char	ccdsec[SZ_LINE]
int	cx1, cx2, cxs, cy1, cy2, cys
int	ccx1, ccx2, ccy1, ccy2
char	trimsec[SZ_LINE]
int	tx1, tx2, txs, txskip1, txskip2
int	ty1, ty2, tys, tyskip1, tyskip2
int	ttx1, ttx2, tty1, tty2
char	biassec[SZ_LINE]
int	bx1, bx2, bxs, bxskip1, bxskip2
int	by1, by2, bys, byskip1, byskip2
int	bbx1, bbx2, bby1, bby2

bool	streq()

begin

	# Get input image dimensions.
	nx  = IM_LEN(im, 1)
	ny  = IM_LEN(im, 2)

	# Get datasec, trimsec and biassec parameters
	call hdmgstr (im, "datasec", datasec, SZ_LINE)
	dx1 = 1
	dx2 = nx
	dxs = 1
	dy1 = 1
	dy2 = ny
	dys = 1
	call ccd_section (datasec, dx1, dx2, dxs, dy1, dy2, dys)
	xdata = dx2 - dx1 + 1
	ydata = dy2 - dy1 + 1

	call clgstr ("ccdproc.trimsec", trimsec, SZ_LINE)
	if (streq (trimsec, "image")) {
	    call hdmgstr (im, "trimsec", trimsec, SZ_LINE)
	}
	tx1 = dx1
	tx2 = dx2
	txs = 1
	ty1 = dy1
	ty2 = dy2
	tys = 1
	call ccd_section (trimsec, tx1, tx2, txs, ty1, ty2, tys)
	txskip1 = tx1 - dx1
	# KLUDGE!
	if ((dx1 == 1 ) && (dx2 == nx)) {	# Bozo datasec assume trimsec
	    dx2 = tx2 + txskip1 		# has equall margins in x
	    xdata = dx2 - dx1 + 1
	}
	txskip2 = dx2 - tx2
	tyskip1 = ty1 - dy1
	tyskip2 = dy2 - ty2

	call clgstr ("ccdproc.biassec", biassec, SZ_LINE)
	if (streq (biassec, "image"))
	    call hdmgstr (im, "biassec", biassec, SZ_LINE)
	bx1 = dx2 + 1
	bx2 = nx
	bxs = 1
	by1 = 1
	by2 = ny
	bys = 1
	call ccd_section (biassec, bx1, bx2, bxs, by1, by2, bys)
	bxskip1 = bx1 - dx2 - 1
	bxskip2 = nx  - bx2
	byskip1 = by1 - dy1
	byskip2 = ny  - by2
	xover   = nx  - xdata
	yover   = ny

	call hdmgstr (im, "ccdsec", ccdsec, SZ_LINE)
	cx1 = dx1
	cx2 = dx2
	cxs = 1
	cy1 = dy1
	cy2 = dy2
	cys = 1
	call ccd_section (ccdsec, cx1, cx2, cxs, cy1, cy2, cys)

	return

entry	quadsection (quad, quadrant)

	# Set trimsec, biassec etc, for quadrant
	switch (quadrant) {
	case 1:
	    ddx1 = dx1
	    ddx2 = ddx1 + xdata / 2 - 1
	    ttx1 = ddx1 + txskip1
	    ttx2 = ddx2 
	    bbx1 = ddx2 + bxskip1 + 1
	    bbx2 = ddx2 + xover / 2 - bxskip2
	    ccx1 = cx1
	    ccx2 = cx1  + xdata / 2 - 1

	    ddy1 = dy1
	    ddy2 = ddy1 + ydata / 2 - 1
	    tty1 = ddy1 + tyskip1
	    tty2 = ddy2
	    bby1 = ddy1 + byskip1 
	    bby2 = ddy2
	    ccy1 = cy1
	    ccy2 = cy1  + ydata / 2 - 1

	case 2:
	    bbx1 = bxskip2 + 1
	    bbx2 = xover / 2 - bxskip1 
	    ddx1 = xover / 2 + 1
	    ddx2 = ddx1 + xdata / 2 - 1
	    ttx1 = ddx1
	    ttx2 = ddx2 - txskip2 
	    ccx1 = cx1 +  xdata / 2
	    ccx2 = ccx1 +  xdata / 2 - 1

	    ddy1 = dy1
	    ddy2 = ddy1 + ydata / 2 - 1
	    tty1 = ddy1 + tyskip1
	    tty2 = ddy2
	    bby1 = ddy1 + byskip1 
	    bby2 = ddy2
	    ccy1 = cy1
	    ccy2 = cy1  + ydata / 2 - 1

	case 3:
	    ddx1 = dx1
	    ddx2 = ddx1 + xdata / 2 - 1
	    ttx1 = ddx1 + txskip1
	    ttx2 = ddx2 
	    bbx1 = ddx2 + bxskip1 + 1
	    bbx2 = ddx2 + xover / 2 - bxskip2
	    ccx1 = cx1
	    ccx2 = cx1  + xdata / 2 - 1

	    ddy1 = 1
	    ddy2 = ddy1 + ydata / 2 - 1
	    tty1 = 1
	    tty2 = ddy2 - tyskip2
	    bby1 = 1
	    bby2 = ddy2 - byskip2
	    ccy1 = cy1  + ydata / 2
	    ccy2 = cy2

	case 4:
	    bbx1 = bxskip2 + 1
	    bbx2 = xover / 2 - bxskip1
	    ddx1 = xover / 2 + 1
	    ddx2 = ddx1 + xdata / 2 - 1
	    ttx1 = ddx1
	    ttx2 = ddx2 - txskip2 
	    ccx1 = cx1 +  xdata / 2
	    ccx2 = ccx1 +  xdata / 2 - 1

	    ddy1 = 1
	    ddy2 = ddy1 + ydata / 2 - 1
	    tty1 = 1
	    tty2 = ddy2 - tyskip2
	    bby1 = 1
	    bby2 = ddy2 - byskip2
	    ccy1 = cy1  + ydata / 2
	    ccy2 = cy2

	}

	call sprintf (datasec, SZ_LINE, "[%d:%d,%d:%d]")
	    call pargi (ddx1)
	    call pargi (ddx2)
	    call pargi (ddy1)
	    call pargi (ddy2)
	call hdmpstr (quad, "datasec", datasec)

	call sprintf (trimsec, SZ_LINE, "[%d:%d,%d:%d]")
	    call pargi (ttx1)
	    call pargi (ttx2)
	    call pargi (tty1)
	    call pargi (tty2)
	call hdmpstr (quad, "trimsec", trimsec)

	call sprintf (biassec, SZ_LINE, "[%d:%d,%d:%d]")
	    call pargi (bbx1)
	    call pargi (bbx2)
	    call pargi (bby1)
	    call pargi (bby2)
	call hdmpstr (quad, "biassec", biassec)

	call sprintf (ccdsec, SZ_LINE, "[%d:%d,%d:%d]")
	    call pargi (ccx1)
	    call pargi (ccx2)
	    call pargi (ccy1)
	    call pargi (ccy2)
	call hdmpstr (quad, "ccdsec", ccdsec)

	return
end

